import java.awt.*;
import java.awt.event.*;

public class Cue implements MouseListener, MouseMotionListener
{
    private R2Vector tip;  // Position vector of tip.
	private double length, angle;
    private Ball cueBall;

	// These are used when the cue is released by the user.
	private double speed = 0;		// Speed after cue is released.
	private double distance = 0;	// Distance from the surface of the cue ball.

    public Cue(double len, double ang, Ball cueball)
    {
        length = len;   angle = ang;
        cueBall = cueball;
        computeTip(0);
    }

    // Computes and sets the coords of the tip so that the tip will be distance dist from
    // the surface of the cue ball. Assumes angle is defined.
    private void computeTip(double dist)
    {
		dist = dist + cueBall.getRadius();
		tip = cueBall.getCenter().plus( R2Vector.dirVec(angle).times(dist) );
    }

	// If the cue has been pulled back, this moves it back towards the cue ball.
	public void move(){
		if (distance > 0){
			speed = speed + 1;
			distance = distance - speed;
			if (distance < 0){ // Cue has hit the cue ball.
				cueBall.setVelocity(R2Vector.dirVec(angle).times(-speed/3));
				distance = 0;  speed = 0;
			}
			computeTip(distance);
		}
		else if (cueBall.getVelocity().length() > 0) computeTip(0);
	}

    public void draw(Graphics g)
    {
		if (cueBall.getVelocity().length() == 0){
	        g.setColor(Color.black);
			R2Vector dirVec = R2Vector.dirVec(angle);
			R2Vector end = tip.plus(dirVec.times(length));
			R2Vector dirVecPerp = dirVec.perp();
			R2Vector end1 = end.plus(dirVecPerp);
			R2Vector end2 = end.minus(dirVecPerp);

	        int tipx = (int)(Math.round(tip.x));	int tipy = (int)(Math.round(tip.y));
    	    int endx = (int)(Math.round(end.x));	int endy = (int)(Math.round(end.y));
        	int end1x = (int)(Math.round(end1.x));	int end1y = (int)(Math.round(end1.y));
	        int end2x = (int)(Math.round(end2.x));	int end2y = (int)(Math.round(end2.y));

    	    g.drawLine(tipx,tipy,endx,endy);
        	g.drawLine(tipx,tipy,end1x,end1y);
	        g.drawLine(tipx,tipy,end2x,end2y);
		}
    }

    // Distance to surface of cue ball.
    private double distToSurface(R2Vector P)
      { return cueBall.getCenter().minus(P).length() - cueBall.getRadius(); }

	// If the mouse is pressed close enough to the cue ball, the cue is aligned with
	// the mouse.
    public void mousePressed(MouseEvent e){
		R2Vector mousePosition = new R2Vector(e.getX(), e.getY());
        if (distToSurface(mousePosition) < length + cueBall.getRadius()) 
        {
            angle = R2Vector.angle(mousePosition.minus(cueBall.getCenter()));
            computeTip(0);
        }
    }

    private boolean pullingBack = false;

    public void mouseDragged(MouseEvent e){
		R2Vector mousePosition = new R2Vector(e.getX(), e.getY());
        double dist = distToSurface(mousePosition);
        if (dist < length) 
            { pullingBack = true;  mousePressed(e); }
        else
            if (pullingBack) 
			{
	            angle = R2Vector.angle(mousePosition.minus(cueBall.getCenter()));
				computeTip(dist - length);
			}
    }

    public void mouseReleased(MouseEvent e){
        if (pullingBack)
        {
            pullingBack = false;
			distance = distToSurface(tip);
        }
    }

    public void mouseClicked(MouseEvent e){}
    public void mouseEntered(MouseEvent e){}
    public void mouseExited(MouseEvent e){}
    public void mouseMoved(MouseEvent e){}

}
